@@ -24,9 +24,10 @@
 #define KALARM_KARECURRENCE_H
 
 #include "kalarmcal_export.h"
-
+#include <KDateTime>
 #include <kcalcore/recurrencerule.h>
 #include <kcalcore/duration.h>
+#include <kcalcore/incidencebase.h>
 namespace KCalCore
 {
 class Recurrence;
@@ -421,7 +422,7 @@ public:
      * @param oldSpec the time specification which provides the clock times
      * @param newSpec the new time specification
      */
-    void shiftTimes(const KDateTime::Spec &oldSpec, const KDateTime::Spec &newSpec);
+    void shiftTimes(const QTimeZone &oldSpec, const QTimeZone &newSpec);
 
     KCalCore::RecurrenceRule *defaultRRuleConst() const;
     /** Return the recurrence's period type. */
